"""
Tests for the data_download module

Includes three tests for the each of the two data functions
"""

import unittest

from detention_data_dashboard.data_download import *

class TestDashboard(unittest.TestCase):
    """
    create class for unittests
    """

    def test_smoke_data1(self):
        """
        Simple smoke test to make sure the data downloaded is a dataframe
        """
        fob = data_download_reg("West Coast")
        self.assertTrue(str(type(fob)) == "<class 'pandas.core.frame.DataFrame'>")

    def test_edge_data1(self):
        """
        Edge test to make sure data will not download if given bad inputs
        """
        with self.assertRaises(NameError):
            data_download_reg("jumping_jacks")

    def test_pattern_data1(self):
        """
        Pattern test for data to make sure download has the correct information included
        (eg correct number of rows and columns)
        """
        for location in ["East Coast", "West Coast", "Southwest", "Midwest", "All"]:
            fob = data_download_reg(location)
            self.assertTrue(fob.shape == (4, 4))

    def test_smoke_data2(self):
        """
        Simple smoke test to make sure the data downloaded is a dataframe
        """
        fob = data_download_arrests_aor("LOS")
        self.assertTrue(str(type(fob)) == "<class 'pandas.core.frame.DataFrame'>")

    def test_edge_data2(self):
        """
        Edge test to make sure data  will not download if given bad inputs
        """
        with self.assertRaises(NameError):
            data_download_arrests_aor("jumping_jacks")

    def test_pattern_data2(self):
        """
        Pattern test for data to make sure download has the correct information included
        (eg correct number of rows and columns)
        """

        for location in ['ATL', 'BAL', 'BOS', 'BUF', 'CHI', 'DAL', 'DEN', 'DET', 'ELP',
                         'HOU', 'HQ', 'LOS', 'MIA', 'NEW', 'NOL','NYC', 'PHI', 'PHO',
                         'SEA', 'SFR', 'SLC', 'SNA', 'SND', 'SPM', 'WAS']:
            fob = data_download_arrests_aor(location)
            self.assertTrue(fob.shape == (4, 2))



"""
Tests for the figure module

Includes one smoke test, one-shot test, and edge tes
"""

import os
import unittest
import detention_data_dashboard

from detention_data_dashboard.figure import *
from detention_data_dashboard.data_download import *
from matplotlib.testing.compare import compare_images

data_path = os.path.join(detention_data_dashboard.__path__[0], 'tests')

class TestDashboard(unittest.TestCase):
    """
    create class for unittests
    """

    def test_smoke_figure1(self):
        """
        Simple smoke test to make sure object is created of the right type
        """
        data_wc = data_download_reg("West Coast")
        fob = display_reg_plot(data_wc)
        self.assertTrue(str(type(fob)) == "<class 'plotly.graph_objs._figure.Figure'>")

    def test_edge_figure1(self):
        """
        Edge test to make sure figure will not display if given bad inputs
        """
        with self.assertRaises(NameError):
            display_reg_plot("jumping_jacks")

    def test_oneshot_figure1(self):
        """
        One shot test for figure by using fuzzy comparison of figure with known results
        """
        for location in ["East Coast", "West Coast", "Southwest", "Midwest", "All"]:
            data = data_download_reg(location)
            temp = display_reg_plot(data)
            temp.write_image(data_path + "/test_images/" + location + "1.png")
            image_a = data_path + "/test_images/" + location + "1" + ".png"
            image_b = data_path + "/test_images/" + location + ".png"
            self.assertTrue(str(type(compare_images(image_a, image_b, tol=.1))) ==
                            "<class 'NoneType'>")

    def test_smoke_figure2(self):
        """
        Simple smoke test to make sure object is created of the right type
        """
        data_los = data_download_arrests_aor("LOS")
        fob = display_aor_arrests_plot(data_los)
        self.assertTrue(str(type(fob)) == "<class 'plotly.graph_objs._figure.Figure'>")

    def test_edge_figure2(self):
        """
        Edge test to make sure figure will not display if given bad inputs
        """
        with self.assertRaises(NameError):
            display_aor_arrests_plot("jumping_jacks")

    def test_oneshot_figure2(self):
        """
        One shot test for figure by using fuzzy comparison of figure with known results
        """
        for location in ['ATL', 'BAL', 'BOS', 'BUF', 'CHI', 'DAL', 'DEN', 'DET', 'ELP',
                         'HOU', 'HQ', 'LOS', 'MIA', 'NEW', 'NOL','NYC', 'PHI', 'PHO',
                         'SEA', 'SFR', 'SLC', 'SNA', 'SND', 'SPM', 'WAS']:
            data = data_download_arrests_aor(location)
            temp = display_aor_arrests_plot(data)
            temp.write_image(data_path + "/test_images/" + location + "1.png")
            image_a = data_path + "/test_images/" + location + "1" + ".png"
            image_b = data_path + "/test_images/" + location + ".png"
            self.assertTrue(str(type(compare_images(image_a, image_b, tol=.1))) ==
                                     "<class 'NoneType'>")



"""
File containing relevant functions for obtaining and downloading data.
"""

import pandas as pd

def data_download_reg(us_loc):
    """
    Generates pandas dataframe of encounters, removals, and arrests per region per fiscal year

    parameters:
        us_loc: a string of a region in the US: West Coast, East Coast, Midwest, Southwest, or All
    returns:
        data_frame: a pandas dataframe containing information about encounters, removals, and
            arrests per region per fiscal year
    """
    if us_loc == "West Coast":
        reg = ['LOS', 'SEA',  'SFR', 'SND']
    elif us_loc == "East Coast":
        reg = ['ATL', 'BAL', 'BOS', 'BUF', 'DET',  'MIA', 'NEW', 'NOL', 'NYC', 'PHI', 'WAS', 'HQ']
    elif us_loc == "Midwest":
        reg = ['CHI', 'SPM']
    elif us_loc == "Southwest":
        reg = ['DAL', 'DEN', 'ELP', 'HOU', 'PHO',  'SLC', 'SNA']
    elif us_loc == "All":
        reg = ['ATL', 'BAL', 'BOS', 'BUF', 'CHI', 'DAL', 'DEN', 'DET', 'ELP', 'HOU', 'HQ', 'LOS',
            'MIA', 'NEW', 'NOL','NYC', 'PHI', 'PHO', 'SEA',
            'SFR', 'SLC', 'SNA', 'SND', 'SPM','WAS']
    else:
        raise NameError('Please enter in a valid US region')

    arrests_by_fy = pd.read_csv("./data/arrests_by_fy.csv")
    encounters_by_fy = pd.read_csv("./data/encounters_by_fy.csv")
    removals_by_fy = pd.read_csv("./data/removals_by_fy.csv")
    date = encounters_by_fy['event_date'].values.tolist()
    enc = encounters_by_fy[reg]
    rem = removals_by_fy[reg]
    arr = arrests_by_fy[reg]
    ind = arr.index.tolist()
    cols = ['date', 'encounters', 'removals', 'arrests']

    data_list = []
    for i in ind:
        data_list.append([date[i], sum(enc.iloc[i]), sum(rem.iloc[i]), sum(arr.iloc[i])])
    data_frame = pd.DataFrame(data = data_list, columns = cols)

    return data_frame

def data_download_arrests_aor(aor):
    """
    Generates pandas dataframe containing information about arrests per aor per fiscal year

    parameters:
        aor: a string of the area of responsibility region code
    returns:
        data_frame: a pandas dataframe containing information about arrests per aor per fiscal year
    """
    arrests = pd.read_csv("./data/arrests_by_fy.csv")
    aor_list = ['ATL', 'BAL', 'BOS', 'BUF', 'CHI', 'DAL', 'DEN', 'DET', 'ELP', 'HOU', 'HQ', 'LOS',
     'MIA','NEW', 'NOL','NYC', 'PHI', 'PHO', 'SEA', 'SFR', 'SLC', 'SNA', 'SND', 'SPM', 'WAS']

    if aor in aor_list:
        arrests_aor = arrests[aor]
        date = arrests['apprehension_date'].values.tolist()
        ind = arrests.index.tolist()
        cols = ['date', 'arrests']

        data_list = []
        for i in ind:
            data_list.append([date[i], arrests_aor[i]])
        data_frame = pd.DataFrame(data = data_list, columns = cols)

        return data_frame

    raise NameError('Please enter in a valid AOR')

def data_download_ice_detention():
    """
    Generates pandas dataframe containing information about the location of ice detention centers

    returns:
        data_frame: a pandas dataframe containing information about the location of ice detention
        centers
    """
    data_frame = pd.read_csv("./data/lat_long_det_fac_99.csv")

    return data_frame



"""
File containing relevant functions for generating figures.
"""

import plotly.express as px
import plotly.graph_objects as go

def display_reg_plot(data_frame):
    """
    Generates plotly figure of encounters, removals, and arrests per region per fiscal year

    parameters:
        data_frame: a pandas dataframe containing information about encounters, removals, and
        arrests per region per fiscal year
    returns:
        fig: a plotly line plot figure of the encounters, removals, and arrests per region per
        fiscal year
    """
    if str(type(data_frame)) == "<class 'pandas.core.frame.DataFrame'>":
        fig = px.line(data_frame, x=data_frame['date'], y=[data_frame['encounters'],
                  data_frame['removals'], data_frame['arrests']],
                  title = "Encounters, Removals, and Arrests in Region per FY",
                  labels=dict(x="Fiscal Year", y="Total"))
        fig.update_xaxes(title="Fiscal Year", nticks = 4)
        fig.update_yaxes(title="Total")
        fig.update_layout(legend_title_text=None)

        return fig

    raise NameError('Please enter in a dataframe')


def display_aor_arrests_plot(data_frame):
    """
    Generates plotly figure of arrests per aor per fiscal year

    parameters:
        data_frame: a pandas dataframe containing information about arrests per aor per fiscal year
    returns:
        fig: a plotly line plot figure of the arrests per aor per fiscal year
    """
    if str(type(data_frame)) == "<class 'pandas.core.frame.DataFrame'>":
        fig = px.line(data_frame, x=data_frame['date'],
                  y=data_frame['arrests'],
                  title = "Arrests in AOR per FY",
                  labels=dict(x="Fiscal Year", y="Total Arrests"))
        fig.update_xaxes(title="Fiscal Year", nticks = 4)
        fig.update_yaxes(title="Total Arrests")
        fig.update_layout(legend_title_text='AOR')

        return fig

    raise NameError('Please enter in a dataframe')

def display_ice_detention_map(data_frame, color):
    """
    Generates plotly map of ice detention centers in the US

    parameters:
        data_frame: a pandas dataframe containing latitudes, longitudes, and addresses of the
        ice detention centers in the us
        color: a string with the color of the dots on the map
    returns:
        fig: a plotly map of ice detention centers in the US
    """

    fig = go.Figure(
        data=go.Scattergeo(
            lon = data_frame['long'],
            lat = data_frame['lat'],
            text = data_frame['address'],
            mode = 'markers',
            marker_color = color,
            ))
    fig.update_layout(
        title = 'ICE Detention Facilities across the US',
        geo_scope='usa',
    )
    return fig



import setuptools

setuptools.setup(
    name="detention_data_dashboard",
    version="0.0.1",
    author="Lucas Olson, Trung-Anh Nguyen, Maddie Gaumer",
    email="lkolson@uw.edu",
    description="a dashboard of ICE enforcement data",
    install_requires=['docutils>=0.3'],
    long_description="a dashboard that provides data on encounters, arrests, and removals by fiscal year between 2016-2019",
    long_description_content_type="text/markdown",
    url="https://github.com/detentiondatadashboard/detention_data_dashboard",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
